'use strict';

const MagicString = require('magic-string');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e["default"] : e; }

const MagicString__default = /*#__PURE__*/_interopDefaultLegacy(MagicString);

async function compileApplet(body, ctx, options = {}) {
  const {
    classPrefix = "uno-",
    hashFn = hash,
    layer = "applet_shortcuts"
  } = options;
  const { uno, tokens } = ctx;
  const replacements = [];
  const result = await Promise.all(body.split(/\s+/).filter(Boolean).map(async (i) => [i, !!await uno.parseToken(i)]));
  const known = result.filter(([, matched]) => matched).map(([i]) => i);
  const unknown = result.filter(([, matched]) => !matched).map(([i]) => i);
  replacements.push(...unknown);
  body = known.join(" ");
  if (body) {
    const hash2 = hashFn(body);
    const className = `${classPrefix}${hash2}`;
    replacements.unshift(className);
    uno.config.shortcuts.push([className, body, { layer }]);
    tokens.add(className);
  }
  return replacements;
}
function hash(str) {
  let i;
  let l;
  let hval = 2166136261;
  for (i = 0, l = str.length; i < l; i++) {
    hval ^= str.charCodeAt(i);
    hval += (hval << 1) + (hval << 4) + (hval << 7) + (hval << 8) + (hval << 24);
  }
  return `00000${(hval >>> 0).toString(36)}`.slice(-6);
}

function transformerApplet(options = {}) {
  const charReg = /[.:%!#()[\/\],]/;
  const classRE = /:?(hover-)?class=\".*?\"/g;
  const string1RE = /([']).*?(['])/g;
  const string2RE = /([\`]).*?([\`])/g;
  return {
    name: "transformer-applet",
    enforce: "pre",
    async transform(s, _, ctx) {
      const code = new MagicString__default(s.toString());
      const classMatches = [...code.original.matchAll(classRE)];
      for (const match of classMatches) {
        if (/\?.*:/g.test(match[0]))
          continue;
        if (/{.+:.+}/g.test(match[0]))
          continue;
        const start = match.index;
        const matchSplit = match[0].split("class=");
        const body = matchSplit[1].slice(1, -1);
        if (charReg.test(body)) {
          const replacements = await compileApplet(body, ctx, options);
          code.overwrite(start, start + match[0].length, `${matchSplit[0]}class="${replacements.join(" ")}"`);
        }
      }
      const string1Matches = [...code.original.matchAll(string1RE)];
      for (const match of string1Matches) {
        if (/\.(png|jpg|jpeg|gif|svg)/g.test(match[0]))
          continue;
        if (/http(s)?:\/\//g.test(match[0]))
          continue;
        const start = match.index;
        const body = match[0].slice(1, -1);
        if (charReg.test(body)) {
          const replacements = await compileApplet(body, ctx, options);
          code.overwrite(start, start + match[0].length, `'${replacements.join(" ")}'`);
        }
      }
      const string2Matches = [...code.original.matchAll(string2RE)];
      for (const match of string2Matches) {
        if (/\$\{.*\}/g.test(match[0]))
          continue;
        const start = match.index;
        const body = match[0].slice(1, -1);
        if (charReg.test(body)) {
          const replacements = await compileApplet(body, ctx, options);
          code.overwrite(start, start + match[0].length, `'${replacements.join(" ")}'`);
        }
      }
      s.overwrite(0, s.original.length, code.toString());
    }
  };
}

module.exports = transformerApplet;
